import json

from idcloud.enums.status import Status
from idcloud.resources.connect_verify_document import ConnectVerifyDocumentResource
from idcloud.responses.scenario import Scenario
from idcloud.utils import JSONEncoder


class ConnectVerifyDocumentFaceResource(ConnectVerifyDocumentResource):
    NAME = 'Connect_Verify_Document_Face'

    def face_match(self, scenario_id: str, face: str):
        payload = {
            'name': self.NAME,
            'input': {
                'face': face
            }
        }

        response = self.requestor.request(
            'PATCH', f'/scs/v1/scenarios/{scenario_id}/state/steps/faceMatch',
            payload=payload
        )

        return Scenario(
            id=response.get('id'),
            name=response.get('name'),
            status=Status(response.get('status').lower()),
            steps=response['state'].get('steps'),
            result=response['state'].get('result'),
            raw_request=json.loads(json.dumps(payload, cls=JSONEncoder)),
            raw_response=response
        )


class ConnectVerifyDocumentFacePassiveResource(ConnectVerifyDocumentFaceResource):
    NAME = 'Connect_Verify_Document_Face_Passive_Liveness'


class ConnectVerifyDocumentFaceEnhancedResource(ConnectVerifyDocumentFaceResource):
    NAME = 'Connect_Verify_Document_Face_Enhanced_Liveness'

    def enhanced_liveness(
        self,
        scenario_id: str,
        frames: list,
        client_device_brand: str = None,
        client_device_model: str = None,
        client_os_version: str = None,
        client_version: str = None,
        localization: str = None,
        programming_language_version: str = None
    ):
        meta_data = self._fill_content(
            client_device_brand=client_device_brand,
            client_device_model=client_device_model,
            client_os_version=client_os_version,
            client_version=client_version,
            localization=localization,
            programming_language_version=programming_language_version
        )

        payload = {
            'name': self.NAME,
            'input': {
                'enhancedLiveness': {
                    'video': {
                        'meta_data': meta_data,
                        'workflow_data': {
                            'frames': frames
                        }
                    }
                }
            }
        }

        response = self.requestor.request(
            'PATCH', f'/scs/v1/scenarios/{scenario_id}/state/steps/enhancedLiveness',
            payload=payload
        )

        return Scenario(
            id=response.get('id'),
            name=response.get('name'),
            status=Status(response.get('status').lower()),
            steps=response['state'].get('steps'),
            result=response['state'].get('result'),
            raw_request=json.loads(json.dumps(payload, cls=JSONEncoder)),
            raw_response=response
        )



import json
from http import HTTPStatus
from typing import Union, Dict

from requests import Request, Session, RequestException, HTTPError
from requests.exceptions import SSLError

from .configuration import Configuration
from .errors import IdCloudException, ForbiddenError
from .utils import JSONEncoder
from . import version


class Requestor(object):
    def __init__(self, configuration: Configuration):
        self._user_agent = f"idcloud_python/{version.__version__}"
        self._configuration = configuration
        self._session = Session()

        if self._configuration:
            self._session.cert = self._configuration.mtls

    def request(
        self, method: str, endpoint_url: str, payload: dict = None, query: dict = None, parse: bool = True
    ) -> Union[str, Dict]:
        req = Request(
            method=method,
            url=self._configuration.environment.api_url + endpoint_url,
            headers={
                "User-Agent": self._user_agent,
                "X-Api-Key": self._configuration.api_key,
                "Authorization": f"Bearer {self._configuration.jwt}"
            },
        )

        if payload and isinstance(payload, dict):
            req.data = json.dumps(payload, cls=JSONEncoder)
            req.headers['Content-Type'] = 'application/json'

        if query and isinstance(query, dict):
            req.params = query

        prepared = req.prepare()

        try:
            r = self._session.send(
                prepared,
                timeout=self._configuration.timeout
            )
        except SSLError as e:
            raise IdCloudException(request=prepared, message='SSL Error', previous=e)
        except RequestException as e:
            raise IdCloudException(request=prepared, message='Request to server timed out.', previous=e)

        try:
            r.raise_for_status()
        except HTTPError as e:
            if e.response.status_code == HTTPStatus.FORBIDDEN:
                raise ForbiddenError(request=prepared, message=e.args[0], previous=e)
            raise IdCloudException(request=prepared, message=e.args[0], previous=e)

        data = r.text

        if parse:
            try:
                data = json.loads(data)
            except json.JSONDecodeError:
                raise IdCloudException(request=prepared, message='Cannot parse response data to json.')

        return data

    def __str__(self) -> str:
        return self._user_agent



from .resources.connect_verify_document import ConnectVerifyDocumentResource
from .resources.connect_verify_document_face import ConnectVerifyDocumentFaceResource, \
    ConnectVerifyDocumentFacePassiveResource, ConnectVerifyDocumentFaceEnhancedResource
from .resources.verify_aml import VerifyAmlResource
from .resources.verify_document import VerifyDocumentResource
from .version import __version__

from .configuration import Configuration
from .requestor import Requestor


class ApiClient(object):
    def __init__(self, configuration: Configuration):
        self._configuration = configuration
        self._requestor = Requestor(configuration)

        self._verify_document = VerifyDocumentResource(self._requestor, self._configuration)
        self._connect_verify_document = ConnectVerifyDocumentResource(self._requestor, self._configuration)
        self._connect_verify_document_face = ConnectVerifyDocumentFaceResource(self._requestor, self._configuration)
        self._connect_verify_document_face_passive = ConnectVerifyDocumentFacePassiveResource(
            self._requestor, self._configuration
        )
        self._connect_verify_document_face_enhanced = ConnectVerifyDocumentFaceEnhancedResource(
            self._requestor, self._configuration
        )
        self._verify_aml = VerifyAmlResource(self._requestor, self._configuration)

    @property
    def version(self) -> str:
        return __version__

    @property
    def requestor(self) -> Requestor:
        return self._requestor

    @property
    def configuration(self) -> Configuration:
        return self._configuration

    @property
    def verify_document(self) -> VerifyDocumentResource:
        return self._verify_document

    @property
    def connect_verify_document(self) -> ConnectVerifyDocumentResource:
        return self._connect_verify_document

    @property
    def connect_verify_document_face(self) -> ConnectVerifyDocumentFaceResource:
        return self._connect_verify_document_face

    @property
    def connect_verify_document_face_passive(self) -> ConnectVerifyDocumentFacePassiveResource:
        return self._connect_verify_document_face_passive

    @property
    def connect_verify_document_face_enhanced(self) -> ConnectVerifyDocumentFaceEnhancedResource:
        return self._connect_verify_document_face_enhanced

    @property
    def verify_aml(self) -> VerifyAmlResource:
        return self._verify_aml

    def __str__(self) -> str:
        return f"{self.requestor}"



from setuptools import setup


def read_files(files):
    data = []
    for file in files:
        with open(file) as f:
            data.append(f.read())
    return "\n".join(data)


meta = {}
with open('idcloud/version.py', encoding='utf-8') as f:
    exec(f.read(), meta)

setup(
    name='idcloud-python',
    version=meta['__version__'],
    packages=[
        'idcloud',
        'idcloud.resources',
        'idcloud.enums',
        'idcloud.responses',
    ],
    install_requires=[
        'python-dotenv==0.18.*',
        'requests==2.26.*',
        'pyjwt==2.1.*'
    ],
    url='https://github.com/idcloud-python',
    license='MIT',
    author='Backbone s.r.o.',
    author_email='office@backbone.sk',
    description='API Client communicating with THALES IdCloud REST API.',
    long_description=read_files(['README.md', 'CHANGELOG.md']),
    long_description_content_type='text/markdown',
    classifiers=[
        # As from https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Database',
        'Topic :: Security'
    ]
)
